#!/bin/sh
#
# Makes a binary distribution for qmail. I hope it 
# works on all GNU/Linux system with little modifications;
# it certainly does on RH.
#  
# On non RH systems, you may have to modify the lines
# in add-account and add-group that updates the NIS 
# database.  (If you are not running a YP server, comment
# the appropriate lines in add-account and add-group)
# 
# Also, useradd may not accept the `-r' flag---get RH's shadow-utils
# package, or modify add-account.
#
# The distribution will be put in the file
# ${PACKAGENAME}-${QMAILVERSION}-${PACKAGEVERSION}.tar.gz


### Configure
RELEASE=$(basename $(pwd)| awk -F'-' '{ print $NF }' )
OS=$(uname -s|tr A-Z a-z)
HW=$(uname -m|tr A-Z a-z)

PACKAGENAME=var-qmail
QMAILVERSION=1.03
PACKAGEVERSION=$RELEASE-gnu-$OS-$HW

# Change this only for experiments.  The final binary package 
# should have QMAILHOME=/var/qmail because all the INSTALL docs
# assume this.

QMAILHOME=/var/qmail

# These show your compilation environment (see BIN.README and,  
# after running this script, $DESTDIR/README)   
# If you plan to distribute the binary package, please
# change PACKAGER, and read
# ftp://koobera.math.uic.edu/www/qmail/dist.html

OSVERSION="OS version: $(uname -sr)"
HARDWARE="Hardware: $(uname -m)"
COMPILER="Compiler: $(gcc -v 2> gcc.v; cat gcc.v | tail -1; rm -f gcc.v)"
LIBRARY="Library: $(rpm -q glibc)"
PACKAGER="Packager: mw@moni.msci.memphis.edu"

# This is needed to be changed in BIN.Makefile (and will appear in 
# $DESTDIR/Makefile )

HOSTNAME='hostname -f'

# The files for the distribution will be installed here

DISTDIR=${PACKAGENAME}-${QMAILVERSION}-${PACKAGEVERSION} 

### EndConfigure
QDIR=qmail-$QMAILVERSION

function Clean() {
 rm -rf $DISTDIR $QDIR
}

Clean
mkdir -p $DISTDIR 

# Needed for installation 
cp BIN.README $DISTDIR/README
cp BIN.Makefile $DISTDIR/Makefile
cp -p add-account add-group $DISTDIR

tar zxvf $QDIR.tar.gz
( 
cd $QDIR 

# create auto_uids.c so that the qmail users
# do not have to be added
cat > auto_uids.c <<EOF
int auto_uida = 0;
int auto_uidd = 0;
int auto_uidl = 0;
int auto_uido = 0;
int auto_uidp = 0;
int auto_uidq = 0;
int auto_uidr = 0;
int auto_uids = 0;
int auto_gidq = 0;
int auto_gidn = 0;
EOF

# Fix Makefile so that auto_uids.c 
# does not get created
sed '/auto_uids.c: /,/^$/ s/^/# /' Makefile > Makefile.tmp
mv Makefile.tmp Makefile

# Fix hier.c so that nothing gets installed
# in man/cat? ---Linux does not use man/cat.
grep -v "man/cat*" hier.c > hier.c.tmp 
mv hier.c.tmp hier.c

echo $QMAILHOME > conf-qmail

make
make man
 
# These are needed/useful after installation 
cp config config-fast dnsfq dnsip dnsptr hostname \
    idedit install instcheck ipmeprint ../$DISTDIR
chmod 755 ../$DISTDIR/*


# Put the files in $DISTDIR
# get a list of files to be put in the distribution
grep "c(" hier.c | awk -F, '{ print $3 }'| sed 's/"//g' > distr_files

cp $(cat distr_files) ../$DISTDIR

# Now find out the byte positions of auto_uid* and auto_gid* in 
# install instcheck qmail-lspawn qmail-queue qmail-rspawn qmail-showctl 
# qmail-start
# just need to find out the byte position of auto_uida.  

# change uids
awk -F"=" '{ $2=($2 + 1); print $1"= "$2";" }' auto_uids.c > auto_uids.c.tmp
mv auto_uids.c.tmp auto_uids.c
make
)

# find byteposition of auto_uida
PROGS="install instcheck qmail-lspawn qmail-queue qmail-rspawn qmail-showctl qmail-start"
for prog in $PROGS; do
    cmp -l $QDIR/$prog $DISTDIR/$prog |
    awk 'NR==1 { print $1 }' > $prog.auto_uida
done

# generate arguments for idedit
bytepos() {
    i=1
    byte=$[ $1 - 1 ]
    while [ $i -le 40 ]; do
	echo -n "$byte "
	byte=$[ byte + 1]
    i=$[ i + 1 ]
    done   
}

for prog in $PROGS; do
    bytepos $(cat $prog.auto_uida) > $prog.idedit_args
done


# Fix $DISTDIR/Makefile
for prog in $PROGS; do
    sed "s/$prog XXX/$prog $(cat $prog.idedit_args)/" $DISTDIR/Makefile \
    > $DISTDIR/Makefile.tmp
    mv $DISTDIR/Makefile.tmp $DISTDIR/Makefile 
done

echo "
HOSTNAME=$HOSTNAME
QMAILHOME=$QMAILHOME
" | ./mem_replace $DISTDIR/Makefile > $DISTDIR/Makefile.tmp
mv $DISTDIR/Makefile.tmp $DISTDIR/Makefile

# Fix $DISTDIR/README
echo "
COMPILER=$COMPILER 
HARDWARE=$HARDWARE
LIBRARY=$LIBRARY
OSVERSION=$OSVERSION
PACKAGER=$PACKAGER
QMAILHOME=$QMAILHOME
" | ./mem_replace $DISTDIR/README > $DISTDIR/README.tmp
mv $DISTDIR/README.tmp $DISTDIR/README 

# Create the distribution
tar zcvf $DISTDIR.tar.gz $DISTDIR

# Cleanup
Clean
for prog in $PROGS; do
    rm -f $prog*
done
